<?php 

namespace CycleGateway;

class Client {
    private $url;
    private $agent_email;
    private $secret_key;
    private $server_secret_key;
    private $redirect_uri;
    // private $api_pass;
    private $verify_ssl = false;
    const API_CREATE_URL = "https://online.cyclebit.net/api/payment/createPayment";
    const API_STATUS_URL = "https://online.cyclebit.net/publicApi/payment/statusPublic/";


    public function __construct($agent_email, $secret_key, $redirect_uri = "", $server_secret_key = "") {
        $this->agent_email = $agent_email;
        $this->secret_key = $secret_key;
        $this->server_secret_key = $server_secret_key;
        $this->redirect_uri = $redirect_uri;
    }


    /**
     * Get Order ID of Gateway. Take care of ?/& issue if still persists.
     *
    */
    public function getGatewayOrderIdFromUriParameters($order_id, $shop_order_id) {
        if ($order_id === NULL && $shop_order_id === NULL) {
            return 0;
        }

        if ( $order_id !== NULL && $order_id != 0 ) {
            return $order_id;
        }
        elseif ( strpos($shop_order_id, 'order_id=') !== false ) {
            $gatewayOrderId = explode('order_id=', $shop_order_id);
            if ( isset($gatewayOrderId[1]) && $gatewayOrderId[1] ) {
                return $gatewayOrderId[1];
            }
        }
        return 0;
    }

    /**
     * Get customer IP address.
    */
    public function getUserIp() {
        if ( ! empty( $_SERVER['HTTP_CLIENT_IP'] ) ) {
            // check ip from share internet
            $ip = $_SERVER['HTTP_CLIENT_IP'];
        }
        elseif ( ! empty( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) {
            // to check ip is pass from proxy
            $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
        }
        else {
            $ip = $_SERVER['REMOTE_ADDR'];
        }
        return $ip;
    }

    /**
     * Create JSON to be sent to the Gateway.
     * 
    */
    public function createOrderJson($gatewayOrder) {

        $orderArray = [
            'description' => $gatewayOrder->orderId,
            'amount' => $gatewayOrder->amount,
            'redirect_url' => $gatewayOrder->redirect_url,
            'callback_url' => $gatewayOrder->callback_url,           
        ];
        foreach ($gatewayOrder->customerArray as $key => $value) {
            if($value){
                $orderArray[$key] = $value;
            }
        }
        return json_encode($orderArray);
    }

    /**
     * Request to URI using curl.
     * $requestParams = [
            'url' => $url,
            'type' => 'POST'|'PUT'|'GET' // default => GET
        ];
     */
    public function request($url, $type = "GET", $data = '') {
        // if (empty($requestParams)) {
        //     return false;
        // }


        $headers = [    
            "content-type: application/json",
            "merchant: " . $this->agent_email,
            "authorization: Bearer " .  $this->getAuthkey()
        ];
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url  ); 

        curl_setopt($ch, CURLOPT_POST, $type == 'POST' ? 1 : 0 );
        if ($type == 'POST') curl_setopt($ch, CURLOPT_POSTFIELDS, $data); 

       // curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, $this->verify_ssl);

        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_TIMEOUT, 1000);

        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1); // TRUE to return the transfer as a string of the return value of curl_exec() instead of outputting it out directly.
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 0);

        curl_setopt($ch, CURLOPT_VERBOSE, true); // Enable this line to see debug prints
        
        $output = curl_exec($ch);
        if (!$output) {
            $output = curl_error($ch);
        }
        $info = curl_getinfo($ch, CURLINFO_HTTP_CODE);


        curl_close($ch);
        return array(
            'code' => $info,
            'body' => $output
        );
    }

    public function getAuthkey()
    {
        $key = $this->server_secret_key;
        $payload = array(
            "email" => $this->agent_email,
            "secret" => $this->secret_key            
        );
        // var_dump($key);
        // var_dump($payload);
        // die();

        return \Firebase\JWT\JWT::encode($payload, $key);
    }

    public function createPayment($requestJSON)
    {
        //$requestJSON =  $this->createOrderJson($data);
        return $this->request(Client::API_CREATE_URL, 'POST', $requestJSON);
    }

    public function getPaymentStatus($transactionId)
    {        
        return $this->request(Client::API_STATUS_URL . $transactionId);
    }

    /**
     * Get status from output body.
     * {
     *      "TransactionId": "e3cc51a0-4542-4433-af5e-28f2dea99e7e",
     *       "State": "completed",
     *       "Success": true
     *  }
     */
    public function getGatewayOrderStatus($gatewayOutputBody) {
        $gatewayOrderStatus = isset($gatewayOutputBody['state']) ? $gatewayOutputBody['state'] : "";
        //$gatewayOrderSuccess = isset($gatewayOutputBody['success']) ? $gatewayOutputBody['success'] : false;
        $gatewayOrderDescription = isset($gatewayOutputBody['description']) ? $gatewayOutputBody['description'] : "";

        return $gatewayOrderStatus . ($gatewayOrderDescription?': '.$gatewayOrderDescription:'');
    }


    

}
